/*
 * Copyright 2008 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 */

#include <iostream>
#include <math.h>

#include <flowdesigner/BufferedNode.h>
#include <flowdesigner/Buffer.h>
#include <flowdesigner/Vector.h>
#include <hark/Source.h>
#include <boost/python.hpp>

using namespace std;
using namespace FD;
using namespace boost::python;

class PyDisplayLocalization;

DECLARE_NODE(PyDisplayLocalization);
/*Node
 *
 * @name PyDisplayLocalization
 * @category HARK:Python
 * @description Display sound streams in the time-azimuth map via boost.python
 *
 * @input_name SOURCES
 * @input_type Vector<ObjectRef>
 * @input_description Source locations with ID. Each element of the vector is a source location with ID specified by "Source".
 *
 * @output_name OUTPUT
 * @output_type Vector<ObjectRef>
 * @output_description The same as input.
 *
 * @parameter_name WINDOW_LENGTH
 * @parameter_type int
 * @parameter_value 1000
 * @parameter_description Window length to show at the same time [sample]
 * 
 * @parameter_name UPDATE_CYCLE
 * @parameter_type int
 * @parameter_value 50
 * @parameter_description The cycle to update the plot.
 *
 * @parameter_name FILENAME
 * @parameter_type string
 * @parameter_value localization.png
 * @parameter_description file name to save. The extension is automatically interpreted.
 * 
 END*/

class PyDisplayLocalization : public BufferedNode {
  int sourcesID;
  int outputID;
  int windowLength;
  int updateCycle;
  string myNodeName;
  string figureName;
  object main_namespace;

public:
  PyDisplayLocalization(string nodeName, ParameterSet params)
    : BufferedNode(nodeName, params)
  {
    sourcesID = addInput("SOURCES");
    outputID = addOutput("OUTPUT");
    
    windowLength = dereference_cast<int>(parameters.get("WINDOW_LENGTH"));
    updateCycle = dereference_cast<int>(parameters.get("UPDATE_CYCLE"));
    figureName = object_cast<string>(parameters.get("FILENAME"));
    if(figureName.size() == 0){
      throw new NodeException(NULL, string("FILENAME is empty."), 
			      __FILE__, __LINE__);
    }
    myNodeName = nodeName;

    inputsCache[sourcesID].lookBack = windowLength;
    inOrder = true;
  }

  ~PyDisplayLocalization()
  {
    try{
      stringstream ss;
      ss << "pylab.savefig('" << figureName << "')" << endl;
      ss << "pylab.close()" << endl;
      exec(ss.str().c_str(), main_namespace);
    } catch(error_already_set){
      PyErr_Print();
    }
  }

  void initialize(void)
  {
    this->BufferedNode::initialize();
    cout << "initialize\n";
    Py_Initialize();
    object main_module(handle<>(borrowed(PyImport_AddModule("__main__"))));
    main_namespace = main_module.attr("__dict__");
    stringstream ss;
    ss << "import pylab" << endl
       << "pylab.ion()" << endl
       << "src = {}" << endl
       << "src.setdefault('" << myNodeName << "', {})" << endl;
    try{
      object ignore = exec(ss.str().c_str(), main_namespace);
    } catch(error_already_set){
      PyErr_Print();
    }
  }


  void calculate(int output_id, int count, Buffer &out)
  {
    RCPtr<Vector<ObjectRef> > pSources = getInput(sourcesID, count);
    out[count] = pSources;

    char str[1000];
    object ignore;

    try{      
      // add srcs
      for(unsigned int i = 0; i < pSources->size(); i++){
	RCPtr<Source> src = (*pSources)[i];
	int angle = (int) (180 / M_PI * atan2(src->x[1], src->x[0]));
	sprintf(str, 
		"src['%s'].setdefault(%d, [])\n"
		"src['%s'][%d].append((%d, %d))\n",
		myNodeName.c_str(), src->id, 
		myNodeName.c_str(), src->id, count, angle);
	ignore = exec(str, main_namespace);
      }
      //delete srcs
      sprintf(str,
	      "for id in src['%s'].keys():\n"
	      "    for idat, dat in enumerate(src['%s'][id]):\n"
	      "        if dat[0] < %d:\n"
	      "            del src['%s'][id][idat]",
	      myNodeName.c_str(),
	      myNodeName.c_str(), count-windowLength, myNodeName.c_str());
      ignore = exec(str, main_namespace);

      if(count % updateCycle == 0){
	//plot srcs
	stringstream ss;
	ss << "tmpsrc = src['" << myNodeName << "']" << endl
	   << "pylab.figure(int('" << myNodeName << "'[-1]))" << endl
	   << "for id in tmpsrc.keys():" << endl
	   << "    pylab.plot([t[0] for t in tmpsrc[id]]," << endl
	   << "               [t[1] for t in tmpsrc[id]]," << endl
	   << "               '-' + 'rgb'[id%3], linewidth=5)" << endl;
	exec(ss.str().c_str(), main_namespace);
	sprintf(str, 
		"pylab.xlabel('Time [frame]')\n"
		"pylab.ylabel('Direction [deg]')\n"
		"pylab.xlim([%d, %d])\n"
		"pylab.ylim([-181, 181])\n"
		"pylab.draw()\n",
		count-windowLength, count);
	ignore = exec(str, main_namespace);
      }
    } catch(error_already_set){
      PyErr_Print();
    }
    
  }    
};
